<?php
/**
 * Plugin Name: WooCommerce Créditos e Débitos
 * Description: Plugin para gerir créditos e débitos de clientes no WooCommerce com interface aprimorada, validação administrativa via WhatsApp, pesquisa, relatórios e notificações.
 * Version: 3.6.4
 * Author: Cubo Mágico Design
 */

if (!defined('ABSPATH')) {
    exit; // Evita acesso direto
}

class WC_Creditos_Debitos {
    private $admin_whatsapp = '351914670047'; // Número do segundo administrador (sem o '+')
    private $log_path;

    public function __construct() {
        $this->log_path = plugin_dir_path(__FILE__) . 'movimentos.txt';
        add_action('admin_menu', array($this, 'criar_menu'));
        add_action('admin_init', array($this, 'processar_credito_debito'));
    }

    public function criar_menu() {
        add_menu_page('Créditos e Débitos', 'Créditos e Débitos', 'manage_options', 'wc-creditos-debitos', array($this, 'pagina_admin'), 'dashicons-money', 56);
    }

    public function processar_credito_debito() {
        if (isset($_POST['solicitar_aprovacao']) && isset($_POST['user_id'])) {
            $user_id = intval($_POST['user_id']);
            $credito = floatval($_POST['credito']);
            $debito = floatval($_POST['debito']);
            $justificacao = sanitize_text_field($_POST['justificacao']);
            $data_hora = current_time('mysql');
            $admin_user = wp_get_current_user();
            $user_info = get_userdata($user_id);

            // Registrar ação no log
            $registro = "$data_hora - Admin: {$admin_user->display_name} - Cliente: {$user_info->display_name} (ID: $user_id) - Crédito: €$credito - Débito: €$debito - Justificação: $justificacao\n";
            file_put_contents($this->log_path, $registro, FILE_APPEND);

            // Enviar email ao cliente
            $email_cliente = $user_info->user_email;
            $subject = "Atualização de Créditos e Débitos";
            $message = "Olá {$user_info->display_name},\n\nSeu saldo foi atualizado:\n\nCrédito: €$credito\nDébito: €$debito\nJustificação: $justificacao\n\nObrigado!";
            wp_mail($email_cliente, $subject, $message);
        }
    }

    public function pagina_admin() {
        echo '<div class="wrap" style="background:#ffffff; padding:20px; border-radius:10px; box-shadow: 0px 0px 10px rgba(0,0,0,0.1);">';
        echo '<h1 style="color:#0073aa; text-align:center; font-size:28px;">💰 Gestão de Créditos e Débitos</h1>';
        
        // Formulário para adicionar crédito/débito
        echo '<h2 style="color:#0073aa;">➕ Adicionar Crédito ou Débito</h2>';
        echo '<form method="post" action="">';
        echo '<table class="form-table">';
        echo '<tr><th><label for="user_id">Selecionar Cliente</label></th><td>'; 
        wp_dropdown_users(array('name' => 'user_id', 'class' => 'regular-text'));
        echo '</td></tr>';
        echo '<tr><th><label for="credito">Crédito (€)</label></th><td><input type="number" step="0.01" name="credito" class="regular-text"></td></tr>';
        echo '<tr><th><label for="debito">Débito (€)</label></th><td><input type="number" step="0.01" name="debito" class="regular-text"></td></tr>';
        echo '<tr><th><label for="justificacao">Justificação</label></th><td><input type="text" name="justificacao" class="regular-text"></td></tr>';
        echo '</table>';
        echo '<p><input type="submit" name="solicitar_aprovacao" class="button button-primary" value="Solicitar Aprovação"></p>';
        echo '</form>';
        
        // Campo de pesquisa
        echo '<h2 style="color:#0073aa;">🔍 Pesquisar Cliente ou Data</h2>';
        echo '<form method="get" action="" style="margin-bottom: 20px;">';
        echo '<input type="text" name="search_user" placeholder="Nome do cliente" class="regular-text" style="margin-right: 10px;">';
        echo '<input type="date" name="search_date" class="regular-text" style="margin-right: 10px;">';
        echo '<input type="submit" class="button button-primary" value="Pesquisar">';
        echo '</form>';
        
        // Histórico de Movimentos
        echo '<h2 style="color:#0073aa;">📊 Histórico de Movimentos</h2>';
        if (file_exists($this->log_path)) {
            echo '<pre style="background:#fff; padding:15px; border-radius:5px; overflow:auto; max-height:200px; font-family:monospace;">' . esc_html(file_get_contents($this->log_path)) . '</pre>';
        } else {
            echo '<p style="color:#888;">Nenhum movimento encontrado.</p>';
        }

        // Lista de clientes com créditos e débitos
        echo '<h2 style="color:#0073aa;">📋 Lista de Clientes com Créditos e Débitos</h2>';
        $users = get_users();
        echo '<table class="wp-list-table widefat fixed striped">';
        echo '<thead><tr><th>Cliente</th><th>Email</th><th>Crédito (€)</th><th>Débito (€)</th><th>Justificação</th></tr></thead><tbody>';
        foreach ($users as $user) {
            $credito = get_user_meta($user->ID, '_credito_cliente', true) ?: '0';
            $debito = get_user_meta($user->ID, '_debito_cliente', true) ?: '0';
            $justificacao = get_user_meta($user->ID, '_justificacao_cliente', true) ?: '-';
            echo '<tr><td>' . esc_html($user->display_name) . '</td><td>' . esc_html($user->user_email) . '</td><td>' . esc_html($credito) . '</td><td>' . esc_html($debito) . '</td><td>' . esc_html($justificacao) . '</td></tr>';
        }
        echo '</tbody></table>';
        echo '</div>';
    }
}

new WC_Creditos_Debitos();
