<?php
/**
 * Plugin Name: WooCommerce Créditos e Débitos
 * Description: Plugin para gerir créditos e débitos de clientes no WooCommerce com interface aprimorada, pesquisa, relatórios e notificações.
 * Version: 3.8.4
 * Author: Cubo Mágico Design
 */

if ( ! defined('ABSPATH') ) {
    exit; // Evita acesso direto
}

// Cria a tabela personalizada na ativação do plugin
register_activation_hook(__FILE__, 'wc_creditos_debitos_create_table');
function wc_creditos_debitos_create_table() {
    global $wpdb;
    $table_name = $wpdb->prefix . 'wc_creditos_debitos_movimentos';
    $charset_collate = $wpdb->get_charset_collate();
    $sql = "CREATE TABLE $table_name (
        id mediumint(9) NOT NULL AUTO_INCREMENT,
        data_hora datetime NOT NULL,
        admin varchar(255) NOT NULL,
        client_name varchar(255) NOT NULL,
        client_id int NOT NULL,
        credit decimal(10,2) NOT NULL,
        debit decimal(10,2) NOT NULL,
        justification text NOT NULL,
        PRIMARY KEY  (id)
    ) $charset_collate;";
    require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
    dbDelta($sql);
}

class WC_Creditos_Debitos {
    private $table_name;
   private $credit_fee_name = 'Crédito a Favor';


public function __construct() {
    global $wpdb;
    $this->table_name = $wpdb->prefix . 'wc_creditos_debitos_movimentos';
    add_action('admin_menu', array($this, 'criar_menu'));
    add_action('admin_init', array($this, 'processar_delete_movimento'));
    add_action('admin_init', array($this, 'processar_credito_debito'));
    add_action('rest_api_init', array($this, 'registrar_api'));
    add_action('admin_enqueue_scripts', array($this, 'enqueue_admin_scripts'));
    // En el constructor, añade también el hook para cancelación:
add_action('woocommerce_order_status_cancelled', array($this, 'revertir_saldo_usuario'), 20, 1);

add_action('woocommerce_calculate_totals', array($this, 'ajustar_fee_credito'), 100, 1);
    // Aplica o saldo no carrinho (igual que antes)
add_action('woocommerce_cart_calculate_fees', array($this, 'aplicar_saldo_compra'), 100, 1);

    // Atualiza o saldo quando a ordem passa para "processing" ou "completed"
    add_action('woocommerce_order_status_processing', array($this, 'actualizar_saldo_usuario'), 20, 1);
    add_action('woocommerce_order_status_completed', array($this, 'actualizar_saldo_usuario'), 20, 1);
       add_action('woocommerce_order_status_on-hold', array($this, 'actualizar_saldo_usuario'), 20, 1);
}



public function ajustar_fee_credito($cart) {
    foreach ( $cart->get_fees() as &$fee ) {
        if ( false !== stripos( $fee->name, $this->credit_fee_name ) ) {
            $fee->taxable = false;
            $fee->tax_class = '';
        }
    }
}


    public function criar_menu() {
        // Página principal
        add_menu_page('Créditos e Débitos', 'Créditos e Débitos', 'manage_options', 'wc-creditos-debitos', array($this, 'pagina_principal'), 'dashicons-money', 56);
        // Subpágina: Movimentos Manuais (exclui os registros do sistema)
        add_submenu_page('wc-creditos-debitos', 'Movimentos Manuais', 'Movimentos Manuais', 'manage_options', 'wc-creditos-debitos-manual', array($this, 'pagina_movimentos_manuais'));
        // Subpágina: Movimentos de Consumo (registros automáticos – não deletáveis)
        add_submenu_page('wc-creditos-debitos', 'Movimentos de Consumo', 'Movimentos de Consumo', 'manage_options', 'wc-creditos-debitos-consumo', array($this, 'pagina_movimentos_consumo'));
    }

    /**
     * Processa a exclusão de movimentos.
     * Apenas permite excluir movimentos manuais (admin <> 'Sistema').
     */
    public function processar_delete_movimento() {
        if ( isset($_GET['delete_mov']) ) {
            $mov_id = intval($_GET['delete_mov']);
            global $wpdb;
            $mov = $wpdb->get_row($wpdb->prepare("SELECT admin FROM {$this->table_name} WHERE id = %d", $mov_id));
            if ( $mov && $mov->admin !== 'Sistema' ) {
                $wpdb->delete($this->table_name, array('id' => $mov_id), array('%d'));
            }
            // Redireciona para a mesma página
            $page = isset($_GET['page']) ? $_GET['page'] : 'wc-creditos-debitos';
            wp_redirect(admin_url('admin.php?page=' . $page));
            exit;
        }
    }

    /**
     * Processa o formulário de adição de crédito/débito e insere o movimento na tabela.
     */
    public function processar_credito_debito() {
        if ( isset($_POST['solicitar_aprovacao']) && isset($_POST['user_id']) ) {
            $user_id      = intval($_POST['user_id']);
            $credito      = floatval($_POST['credito']);
            $debito       = floatval($_POST['debito']);
            $justificacao = sanitize_text_field($_POST['justificacao']);
            $data_hora    = current_time('mysql');
            $admin_user   = wp_get_current_user();
            $user_info    = get_userdata($user_id);

            global $wpdb;
            $wpdb->insert(
                $this->table_name,
                array(
                    'data_hora'     => $data_hora,
                    'admin'         => $admin_user->display_name,
                    'client_name'   => $user_info->display_name,
                    'client_id'     => $user_id,
                    'credit'        => $credito,
                    'debit'         => $debito,
                    'justification' => $justificacao,
                ),
                array(
                    '%s',
                    '%s',
                    '%s',
                    '%d',
                    '%f',
                    '%f',
                    '%s',
                )
            );

            // Atualiza os metadados do usuário (opcional para compatibilidade)
            $atual_credito = floatval(get_user_meta($user_id, '_credito_cliente', true));
            $atual_debito  = floatval(get_user_meta($user_id, '_debito_cliente', true));
            $novo_credito  = $atual_credito + $credito;
            $novo_debito   = $atual_debito + $debito;
            update_user_meta($user_id, '_credito_cliente', $novo_credito);
            update_user_meta($user_id, '_debito_cliente', $novo_debito);
            update_user_meta($user_id, '_justificacao_cliente', $justificacao);
// Llamar a la función de enviar notificación
$this->enviar_notificacion_credito_debito($user_id, $credito, $debito, $justificacao);

        }
    }


public function enviar_notificacion_credito_debito($user_id, $credito, $debito, $justificacion) {
    $user_info = get_userdata($user_id);
    if (!$user_info) return;

    $email = $user_info->user_email;
    $secret_code = 'Samipe2025.';
    
    // Determinar el tipo de movimiento
    if ($credito > 0) {
        $title = "Crédito adicionado à sua conta corrente";
        $message = "Foi adicionado um crédito de €" . number_format($credito, 2, ',', '.') . " à sua conta. Justificação: " . $justificacion;
    } elseif ($debito > 0) {
        $title = "Débito adicionado à sua conta corrente";
        $message = "Foi adicionado um débito de €" . number_format($debito, 2, ',', '.') . " à sua conta. Justificação: " . $justificacion;
    } else {
        return;
    }

    // Datos a enviar al backend
    $data = [
        'email' => $email,
        'title' => $title,
        'message' => $message,
        'secret_code' => $secret_code
    ];

    // URL del endpoint en tu backend Slim
    $url = 'https://app-samipe.pt/api/notifications/sendNotificationCreditDebit';

    // Enviar la notificación al backend
    $response = wp_remote_post($url, [
        'method' => 'POST',
        'body' => $data,
        'timeout' => 10,
    ]);

    // Verificar la respuesta
    if (is_wp_error($response)) {
        error_log("Erro ao enviar a notificação: " . $response->get_error_message());
    } else {
        $body = wp_remote_retrieve_body($response);
    }
}


    /**
     * Página principal: exibe formulário e lista de clientes com saldos.
     */
    public function pagina_principal() {
        global $wpdb;
        echo '<div class="wrap" style="background:#fff; padding:20px; border-radius:10px; box-shadow: 0 0 10px rgba(0,0,0,0.1);">';
        echo '<h1 style="color:#0073aa; text-align:center; font-size:28px;">💰 Gestão de Créditos e Débitos</h1>';

        // Formulário para adicionar crédito/débito
        echo '<h2 style="color:#0073aa;">➕ Adicionar Crédito ou Débito</h2>';
        echo '<form method="post" action="" style="margin-bottom:20px;">';
        echo '<table class="form-table">';
        echo '<tr><th><label for="user_id">Selecionar Cliente</label></th><td>';
        $users_dropdown = get_users();
        usort($users_dropdown, function($a, $b) {
            return strcasecmp($a->display_name, $b->display_name);
        });
 echo '<select id="user_id" name="user_id" style="width:300px;"></select>';
        echo '</td></tr>';
        echo '<tr><th><label for="credito">Crédito (€)</label></th><td><input type="number" step="0.01" name="credito" class="regular-text"></td></tr>';
        echo '<tr><th><label for="debito">Débito (€)</label></th><td><input type="number" step="0.01" name="debito" class="regular-text"></td></tr>';
        echo '<tr><th><label for="justificacao">Justificação</label></th><td><input type="text" name="justificacao" class="regular-text"></td></tr>';
        echo '</table>';
        echo '<p><input type="submit" name="solicitar_aprovacao" class="button button-primary" value="Solicitar Aprovação"></p>';
        echo '</form>';

        // Lista de clientes com saldos
        echo '<h2 style="color:#0073aa; margin-top:40px;">📊 Clientes com Saldos</h2>';
        $clientes = $wpdb->get_results("SELECT client_id, client_name, SUM(credit) as total_credit, SUM(debit) as total_debit FROM {$this->table_name} GROUP BY client_id ORDER BY client_name ASC");
        if ( $clientes ) {
            echo '<table class="wp-list-table widefat fixed striped">';
            echo '<thead><tr>';
            echo '<th>Cliente</th>';
            echo '<th>Email</th>';
            echo '<th>Total Crédito (€)</th>';
            echo '<th>Total Débito (€)</th>';
            echo '<th>Saldo (€)</th>';
            echo '</tr></thead>';
            echo '<tbody>';
            foreach ( $clientes as $cliente ) {
                $user_data = get_userdata($cliente->client_id);
                $email = $user_data ? $user_data->user_email : '-';
                $saldo = floatval($cliente->total_credit) - floatval($cliente->total_debit);
                echo '<tr>';
                echo '<td>' . esc_html($cliente->client_name) . '</td>';
                echo '<td>' . esc_html($email) . '</td>';
                echo '<td>' . esc_html(number_format($cliente->total_credit, 2)) . '</td>';
                echo '<td>' . esc_html(number_format($cliente->total_debit, 2)) . '</td>';
                echo '<td>' . esc_html(number_format($saldo, 2)) . '</td>';
                echo '</tr>';
            }
            echo '</tbody></table>';
        } else {
            echo '<p style="color:#888;">Nenhum cliente com saldo encontrado.</p>';
        }

        echo '<p style="margin-top:20px;">Acesse os <a href="' . esc_url(admin_url('admin.php?page=wc-creditos-debitos-manual')) . '">Movimentos Manuais</a> e os <a href="' . esc_url(admin_url('admin.php?page=wc-creditos-debitos-consumo')) . '">Movimentos de Consumo</a>.</p>';

        echo '</div>';
    }

    /**
     * Página de Movimentos Manuais (registros com admin <> 'Sistema')
     * Permite filtrar, exportar a PDF e excluir os registros.
     */
    public function pagina_movimentos_manuais() {
        global $wpdb;
        echo '<div class="wrap" style="background:#fff; padding:20px; border-radius:10px; box-shadow: 0 0 10px rgba(0,0,0,0.1);">';
        echo '<h1 style="color:#0073aa; text-align:center; font-size:28px;">📋 Movimentos Manuais</h1>';
        
        // Botão de exportação a PDF
 echo '<p><a href="' . esc_url(add_query_arg('export_pdf', 'manual')) . '" class="button" target="_blank">Exportar para PDF</a></p>';
        
        echo '<form method="get" action="" style="margin-bottom:20px;">';
        echo '<input type="hidden" name="page" value="wc-creditos-debitos-manual">';
        echo '<input type="text" name="search_user" placeholder="Cliente (Nome)" class="regular-text" style="margin-right:10px;">';
        echo '<input type="date" name="search_date" class="regular-text" style="margin-right:10px;">';
        echo '<input type="submit" class="button button-primary" value="Filtrar">';
        echo '</form>';

        $where_clauses = array("admin <> 'Sistema'");
        $search_user = isset($_GET['search_user']) ? sanitize_text_field($_GET['search_user']) : '';
        $search_date = isset($_GET['search_date']) ? sanitize_text_field($_GET['search_date']) : '';
        if ( $search_user ) {
            $where_clauses[] = $wpdb->prepare("client_name LIKE %s", '%' . $wpdb->esc_like($search_user) . '%');
        }
        if ( $search_date ) {
            $where_clauses[] = $wpdb->prepare("DATE(data_hora) = %s", $search_date);
        }
        $where_sql = 'WHERE ' . implode(' AND ', $where_clauses);

        // Se for solicitado exportar PDF, gera e envia o arquivo
        if ( isset($_GET['export_pdf']) && $_GET['export_pdf'] == 'manual' ) {
            $movimentos = $wpdb->get_results("SELECT * FROM {$this->table_name} $where_sql ORDER BY data_hora DESC");
            $this->export_pdf($movimentos, "Movimentos Manuais");
            exit;
        }

        $movimentos = $wpdb->get_results("SELECT * FROM {$this->table_name} $where_sql ORDER BY data_hora DESC");
        if ( $movimentos ) {
            echo '<table class="wp-list-table widefat fixed striped">';
            echo '<thead><tr>';
            echo '<th>ID</th>';
            echo '<th>Data/Hora</th>';
            echo '<th>Admin</th>';
            echo '<th>Cliente</th>';
            echo '<th>Crédito (€)</th>';
            echo '<th>Débito (€)</th>';
            echo '<th>Justificação</th>';
            echo '<th>Ações</th>';
            echo '</tr></thead>';
            echo '<tbody>';
            foreach ( $movimentos as $mov ) {
                echo '<tr>';
                echo '<td>' . esc_html($mov->id) . '</td>';
                echo '<td>' . esc_html($mov->data_hora) . '</td>';
                echo '<td>' . esc_html($mov->admin) . '</td>';
                echo '<td>' . esc_html($mov->client_name) . '</td>';
                echo '<td>' . esc_html($mov->credit) . '</td>';
                echo '<td>' . esc_html($mov->debit) . '</td>';
                echo '<td>' . esc_html($mov->justification) . '</td>';
                echo '<td><a href="' . esc_url(admin_url('admin.php?page=wc-creditos-debitos-manual&delete_mov=' . $mov->id)) . '" onclick="return confirm(\'¿Está seguro de eliminar este movimento?\');">Eliminar</a></td>';
                echo '</tr>';
            }
            echo '</tbody></table>';
        } else {
            echo '<p style="color:#888;">Nenhum movimento manual encontrado.</p>';
        }
        echo '</div>';
    }

    /**
     * Página de Movimentos de Consumo (registros com admin = 'Sistema')
     * Apenas exibe e permite filtrar; sem opção de exclusão.
     */
    public function pagina_movimentos_consumo() {
        global $wpdb;
        echo '<div class="wrap" style="background:#fff; padding:20px; border-radius:10px; box-shadow: 0 0 10px rgba(0,0,0,0.1);">';
        echo '<h1 style="color:#0073aa; text-align:center; font-size:28px;">📋 Movimentos de Consumo</h1>';
        
        // Botão de exportação a PDF
 echo '<p><a href="' . esc_url(add_query_arg('export_pdf', 'manual')) . '" class="button" target="_blank">Exportar para PDF</a></p>';
        
        echo '<form method="get" action="" style="margin-bottom:20px;">';
        echo '<input type="hidden" name="page" value="wc-creditos-debitos-consumo">';
        echo '<input type="text" name="search_user" placeholder="Cliente (Nome)" class="regular-text" style="margin-right:10px;">';
        echo '<input type="date" name="search_date" class="regular-text" style="margin-right:10px;">';
        echo '<input type="submit" class="button button-primary" value="Filtrar">';
        echo '</form>';

        $where_clauses = array("admin = 'Sistema'");
        $search_user = isset($_GET['search_user']) ? sanitize_text_field($_GET['search_user']) : '';
        $search_date = isset($_GET['search_date']) ? sanitize_text_field($_GET['search_date']) : '';
        if ( $search_user ) {
            $where_clauses[] = $wpdb->prepare("client_name LIKE %s", '%' . $wpdb->esc_like($search_user) . '%');
        }
        if ( $search_date ) {
            $where_clauses[] = $wpdb->prepare("DATE(data_hora) = %s", $search_date);
        }
        $where_sql = 'WHERE ' . implode(' AND ', $where_clauses);

        // Se solicitado exportar PDF, gera e envia o arquivo
        if ( isset($_GET['export_pdf']) && $_GET['export_pdf'] == 'consumo' ) {
            $movimentos = $wpdb->get_results("SELECT * FROM {$this->table_name} $where_sql ORDER BY data_hora DESC");
            $this->export_pdf($movimentos, "Movimentos de Consumo");
            exit;
        }

        $movimentos = $wpdb->get_results("SELECT * FROM {$this->table_name} $where_sql ORDER BY data_hora DESC");
        if ( $movimentos ) {
            echo '<table class="wp-list-table widefat fixed striped">';
            echo '<thead><tr>';
            echo '<th>ID</th>';
            echo '<th>Data/Hora</th>';
            echo '<th>Cliente</th>';
            echo '<th>Crédito (€)</th>';
            echo '<th>Débito (€)</th>';
            echo '<th>Justificação</th>';
            echo '</tr></thead>';
            echo '<tbody>';
foreach ( $movimentos as $mov ) {
    echo '<tr>';
    echo '<td>' . esc_html($mov->id) . '</td>';
    echo '<td>' . esc_html($mov->data_hora) . '</td>';
    echo '<td>' . esc_html($mov->client_name) . '</td>';
    
    // Para el crédito:
    $credit_val = abs($mov->credit);
    if ( false !== stripos($mov->justification, 'Uso de crédito') ) {
        // Si es "Uso de crédito", se muestra como negativo.
        $credit_display = '-' . number_format($credit_val, 2);
    } elseif ( false !== stripos($mov->justification, 'Reversão da utilização de crédito') ) {
        // Si es reversión, se muestra como positivo.
        $credit_display = '+' . number_format($credit_val, 2);
    } else {
        $credit_display = number_format($credit_val, 2);
    }
    echo '<td>' . esc_html($credit_display) . '</td>';

    // Para el débito:
    $debit_val = abs($mov->debit);
    if ( false !== stripos($mov->justification, 'Uso de débito') ) {
        // Si es "Uso de débito", se muestra como positivo.
        $debit_display = '+' . number_format($debit_val, 2);
    } elseif ( false !== stripos($mov->justification, 'Reversão da utilização do débito') ) {
        // Si es reversión, se muestra como negativo.
        $debit_display = '-' . number_format($debit_val, 2);
    } else {
        $debit_display = number_format($debit_val, 2);
    }
    echo '<td>' . esc_html($debit_display) . '</td>';
    
    echo '<td>' . esc_html($mov->justification) . '</td>';
    echo '</tr>';
}

            echo '</tbody></table>';
        } else {
            echo '<p style="color:#888;">Nenhum movimento de consumo encontrado.</p>';
        }
        echo '</div>';
    }

    /**
     * Função para exportar os movimentos para PDF.
     * É necessário que a biblioteca FPDF esteja disponível na pasta "fpdf" dentro do plugin.
     */
   public function export_pdf($movimentos, $title) {
    if ( ! class_exists('FPDF') ) {
        require_once(plugin_dir_path(__FILE__) . 'fpdf/fpdf.php');
    }
    $pdf = new FPDF();
    $pdf->AddPage();
    // Usamos una fuente que soporte ISO-8859-1
    $pdf->SetFont('Arial','B',16);
    // Convertir el título a ISO-8859-1
    $pdf->Cell(0,10, iconv('UTF-8','ISO-8859-1//TRANSLIT', $title), 0, 1, 'C');
    $pdf->Ln(5);

    // Cabeçalho da tabela
    $pdf->SetFont('Arial','B',10);
    $pdf->Cell(10,10, iconv('UTF-8','ISO-8859-1//TRANSLIT','ID'), 1);
    $pdf->Cell(30,10, iconv('UTF-8','ISO-8859-1//TRANSLIT','Data/Hora'), 1);
    $pdf->Cell(30,10, iconv('UTF-8','ISO-8859-1//TRANSLIT','Admin'), 1);
    $pdf->Cell(40,10, iconv('UTF-8','ISO-8859-1//TRANSLIT','Cliente'), 1);
    $pdf->Cell(20,10, iconv('UTF-8','ISO-8859-1//TRANSLIT','Crédito'), 1);
    $pdf->Cell(20,10, iconv('UTF-8','ISO-8859-1//TRANSLIT','Débito'), 1);
    $pdf->Cell(50,10, iconv('UTF-8','ISO-8859-1//TRANSLIT','Justificação'), 1);
    $pdf->Ln();
    $pdf->SetFont('Arial','',9);
    foreach($movimentos as $mov) {
        $pdf->Cell(10,10, $mov->id, 1);
        $pdf->Cell(30,10, iconv('UTF-8','ISO-8859-1//TRANSLIT', $mov->data_hora), 1);
        $pdf->Cell(30,10, iconv('UTF-8','ISO-8859-1//TRANSLIT', $mov->admin), 1);
        $pdf->Cell(40,10, iconv('UTF-8','ISO-8859-1//TRANSLIT', $mov->client_name), 1);
        $pdf->Cell(20,10, $mov->credit, 1);
        $pdf->Cell(20,10, $mov->debit, 1);
        $pdf->Cell(50,10, iconv('UTF-8','ISO-8859-1//TRANSLIT', $mov->justification), 1);
        $pdf->Ln();
    }
    // Limpia el buffer de salida antes de enviar cabeceras
    if (ob_get_length()) {
        ob_clean();
    }
    flush();
    header('Content-Type: application/pdf');
    // Usamos inline para que se abra en una pestaña nueva
    header('Content-Disposition: inline; filename="'.$title.'.pdf"');
    $pdf->Output();
    exit;
}


    public function registrar_api() {
        register_rest_route('wc/v1', '/user-balance/(?P<id>\d+)', array(
            'methods'             => 'GET',
            'callback'            => array($this, 'obter_user_balance'),
            'permission_callback' => '__return_true'
        ));

        register_rest_route('wc/v1', '/user-history/(?P<id>\d+)', array(
            'methods'             => 'GET',
            'callback'            => array($this, 'obter_user_history'),
            'permission_callback' => '__return_true'
        ));
    }

public function obter_user_balance($request) {
    global $wpdb;
    $user_id = intval($request['id']);
    $result = $wpdb->get_row($wpdb->prepare("SELECT SUM(credit) as total_credit, SUM(debit) as total_debit FROM {$this->table_name} WHERE client_id = %d", $user_id));
    $total_credit = floatval($result->total_credit);
    $total_debit  = floatval($result->total_debit);
    $saldo = $total_credit - $total_debit;
    return new WP_REST_Response(array(
        'credito' => $total_credit,
        'debito'  => $total_debit,
        'saldo'   => $saldo,
    ), 200);
}


    public function obter_user_history($request) {
        $user_id = intval($request['id']);
        global $wpdb;
        $movimentos = $wpdb->get_results($wpdb->prepare("SELECT * FROM {$this->table_name} WHERE client_id = %d ORDER BY data_hora DESC", $user_id));
        return new WP_REST_Response($movimentos, 200);
    }
    
    
  public function revertir_saldo_usuario($order_id) {
    $order = wc_get_order($order_id);
    // Si no se aplicó el ajuste en esta orden, no hacemos nada
    if ( ! $order->get_meta('_saldo_aplicado') ) {
        return;
    }
    $user_id = $order->get_user_id();
    if ( ! $user_id ) return;
    
    $applied_credit = 0;
    $applied_fee    = 0;
    // Recorremos los items de fee para identificar cuánto se aplicó
    foreach ( $order->get_items('fee') as $fee_item ) {
        $name   = $fee_item->get_name();
        $amount = $fee_item->get_total();
        if ( $name == __('Crédito a Favor', 'wc-creditos-debitos') ) {
            $applied_credit = abs($amount);
        } elseif ( $name == __('Saldo Pendente', 'wc-creditos-debitos') ) {
            $applied_fee = $amount;
        }
    }
    
    global $wpdb;
    $user_info = get_userdata($user_id);
    $data_hora = current_time('mysql');
    
    // Si se consumió crédito, se restaura y se inserta un movimiento de reversión
    if ( $applied_credit > 0 ) {
        $current_credit = floatval(get_user_meta($user_id, '_credito_cliente', true));
        $new_credit = $current_credit + $applied_credit;
        update_user_meta($user_id, '_credito_cliente', $new_credit);
        // Inserta registro de reversión de crédito
        $wpdb->insert(
            $this->table_name,
            array(
                'data_hora'     => $data_hora,
                'admin'         => 'Sistema',
                'client_name'   => $user_info->display_name,
                'client_id'     => $user_id,
                'credit'        => $applied_credit,  // valor positivo para restaurar
                'debit'         => 0,
                'justification' => sprintf(__('Reversão da utilização de crédito no pedido #%d', 'wc-creditos-debitos'), $order_id)
            ),
            array('%s','%s','%s','%d','%f','%f','%s')
        );
    }
    // Si se consumió débito, se restaura y se inserta un movimiento de reversión
    if ( $applied_fee > 0 ) {
        $current_debit = floatval(get_user_meta($user_id, '_debito_cliente', true));
        $new_debit = $current_debit + $applied_fee;
        update_user_meta($user_id, '_debito_cliente', $new_debit);
        // Inserta registro de reversión de débito
        $wpdb->insert(
            $this->table_name,
            array(
                'data_hora'     => $data_hora,
                'admin'         => 'Sistema',
                'client_name'   => $user_info->display_name,
                'client_id'     => $user_id,
                'credit'        => 0,
                'debit'         => $applied_fee,  // valor positivo para restaurar
                'justification' => sprintf(__('Reversão da utilização do débito no pedido #%d', 'wc-creditos-debitos'), $order_id)
            ),
            array('%s','%s','%s','%d','%f','%f','%s')
        );
    }
    
    // Elimina la marca para que la reversión no se aplique varias veces
    $order->delete_meta_data('_saldo_aplicado');
    $order->save();
}



    /**
     * Aplica o saldo do usuário no carrinho:
     * - Se tiver saldo a favor, aplica desconto.
     * - Se tiver saldo negativo, adiciona uma taxa.
     */
public function aplicar_saldo_compra($cart) {
    if ( is_admin() && ! defined('DOING_AJAX') ) return;
    if ( ! is_user_logged_in() ) return;

    $user_id = get_current_user_id();

    global $wpdb;
    $result = $wpdb->get_row($wpdb->prepare(
        "SELECT SUM(credit) as total_credit, SUM(debit) as total_debit FROM {$this->table_name} WHERE client_id = %d",
        $user_id
    ));
    $saldo = floatval($result->total_credit) - floatval($result->total_debit);


    if ( $saldo > 0 ) {
        $total_produtos = $cart->get_cart_contents_total();
        $total_envio    = $cart->shipping_total;
        $total_pedido   = $total_produtos + $total_envio;
        $discount = min($saldo, $total_pedido);
        
        // Obtener la tasa de IVA de forma dinámica
        $tax_rates = WC_Tax::get_rates('');
        $tax_rate = 0;
        if ( ! empty($tax_rates) ) {
            $rate = current($tax_rates);
            $tax_rate = floatval($rate['rate']) / 100;
        }

        
        // Convertir el descuento bruto al valor neto
        $discount_net = $discount / (1 + $tax_rate);


        $cart->add_fee( __($this->credit_fee_name, 'wc-creditos-debitos'), -$discount_net, false, '' );
    } elseif ( $saldo < 0 ) {
        $fee = abs($saldo);
        $cart->add_fee( __('Saldo Pendente', 'wc-creditos-debitos'), $fee );
    }
}








    /**
     * Atualiza o saldo do usuário após a compra.
     * Além de atualizar os metadados, insere um registro na tabela de movimentos
     * para registrar o consumo do saldo no pedido.
     */
public function actualizar_saldo_usuario($order_id) {
    $order = wc_get_order($order_id);
    // Verifica se o saldo já foi aplicado
    if ($order->get_meta('_saldo_aplicado')) {
        return;
    }
    $user_id = $order->get_user_id();
    if ( ! $user_id ) return;

    $applied_credit = 0;
    $applied_fee    = 0;
foreach ( $order->get_items('fee') as $fee_item ) {
    $name   = $fee_item->get_name();
    $amount = $fee_item->get_total();
    if ( $name == __('Crédito a Favor', 'wc-creditos-debitos') ) {
        // Obtén la tasa de IVA de forma dinámica
        $tax_rates = WC_Tax::get_rates('');
        $tax_rate = 0;
        if ( ! empty($tax_rates) ) {
            $rate = current($tax_rates);
            $tax_rate = floatval($rate['rate']) / 100;
        }
        // $amount es el valor neto (por ejemplo, 1.42), convertirlo a bruto:
        $applied_credit = abs($amount) * (1 + $tax_rate);
    } elseif ( $name == __('Saldo Pendente', 'wc-creditos-debitos') ) {
        $applied_fee = $amount;
    }
}


    global $wpdb;
    $user_info = get_userdata($user_id);
    $data_hora = current_time('mysql');

    if ( $applied_credit > 0 ) {
        $current_credit = floatval(get_user_meta($user_id, '_credito_cliente', true));
        $new_credit = max(0, $current_credit - $applied_credit);
        update_user_meta($user_id, '_credito_cliente', $new_credit);
        // Registra o consumo de crédito na tabela de movimentos
        $wpdb->insert(
            $this->table_name,
            array(
                'data_hora'     => $data_hora,
                'admin'         => 'Sistema',
                'client_name'   => $user_info->display_name,
                'client_id'     => $user_id,
                'credit'        => -$applied_credit,
                'debit'         => 0,
                'justification' => sprintf(__('Uso de crédito em pedido #%d', 'wc-creditos-debitos'), $order_id)
            ),
            array('%s','%s','%s','%d','%f','%f','%s')
        );
    }
    if ( $applied_fee > 0 ) {
        $current_debit = floatval(get_user_meta($user_id, '_debito_cliente', true));
        $new_debit = max(0, $current_debit - $applied_fee);
        update_user_meta($user_id, '_debito_cliente', $new_debit);
        // Registra o consumo de débito na tabela de movimentos
        $wpdb->insert(
            $this->table_name,
            array(
                'data_hora'     => $data_hora,
                'admin'         => 'Sistema',
                'client_name'   => $user_info->display_name,
                'client_id'     => $user_id,
                'credit'        => 0,
                'debit'         => -$applied_fee,
                'justification' => sprintf(__('Uso de débito em pedido #%d', 'wc-creditos-debitos'), $order_id)
            ),
            array('%s','%s','%s','%d','%f','%f','%s')
        );
    }
    // Marca a ordem para que não seja processada novamente
    $order->update_meta_data('_saldo_aplicado', 'yes');
    $order->save();
}
public function enqueue_admin_scripts($hook) {
    $allowed = array(
        'toplevel_page_wc-creditos-debitos',
        'wc-creditos-debitos_page_wc-creditos-debitos-manual',
        'wc-creditos-debitos_page_wc-creditos-debitos-consumo'
    );
    if ( ! in_array($hook, $allowed) ) {
        return;
    }
    
    // Encolar el script de Select2 (ya registrado o incluido por WordPress)
    wp_enqueue_script('select2');
    
    // Encolar el CSS de Select2 desde un CDN
    wp_enqueue_style('select2-css', 'https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.13/css/select2.min.css', array(), '4.0.13');
    
    // Encolar el script personalizado
    wp_enqueue_script('wccd-custom-script', plugin_dir_url(__FILE__) . 'js/wccd-custom.js', array('jquery', 'select2'), '1.0.0', true);
    wp_localize_script('wccd-custom-script', 'wccd_ajax_object', array(
        'ajax_url' => admin_url('admin-ajax.php'),
        'nonce'    => wp_create_nonce('wccd_user_search_nonce'),
    ));
}



}

function wccd_user_search_ajax() {
    // Verifica el nonce para seguridad
    check_ajax_referer('wccd_user_search_nonce', 'security');
    
    $search_term = isset($_GET['q']) ? sanitize_text_field($_GET['q']) : '';
    
    $args = array(
        'search'         => '*' . $search_term . '*',
        'search_columns' => array('display_name', 'user_email'),
        'number'         => 20,
    );
    
    $user_query = new WP_User_Query($args);
    $results = array();
    
    if ( ! empty( $user_query->get_results() ) ) {
        foreach ( $user_query->get_results() as $user ) {
            $user_picture = '';
            // Llamada a la API externa para obtener la imagen de perfil
            $api_response = wp_remote_get("https://app-samipe.pt/api/getUserPicture?user_id_wp=" . $user->ID);
            if (!is_wp_error($api_response)) {
                $body = json_decode(wp_remote_retrieve_body($api_response));
                if (isset($body->user_picture)) {
                    $user_picture = $body->user_picture;
                }
            }
            $results[] = array(
                'id'      => $user->ID,
                'text'    => $user->display_name . ' - ' . $user->user_email,
                'picture' => $user_picture,
            );
        }
    }
    
    wp_send_json($results);
}
add_action('wp_ajax_wccd_user_search', 'wccd_user_search_ajax');


new WC_Creditos_Debitos();






